﻿using System;
using System.IO;
using System.Windows;
using System.Windows.Controls;

namespace RefApp8
{
    /// <summary>
    /// ログ出力有無を示す列挙値です
    /// </summary>
    public enum OutputStatusBar
    {
        ON,
        OFF,
    }

    /// <summary>
    /// LogWindow.xaml の相互作用ロジック
    /// </summary>
    public partial class LogWindow : Window
    {
        /// <summary>
        /// ログメッセージです
        /// </summary>
        static public string Messages { get; set; }

        /// <summary>
        /// コンストラクタです
        /// </summary>
        public LogWindow()
        {
            InitializeComponent();
            LogText.Text = Messages;
        }

        /// <summary>
        /// ログ出力します
        /// </summary>
        /// <param name="message">ログメッセージ</param>
        /// <param name="outputStatusBar">ステータスバー表示有無</param>
        public void Print(string message, OutputStatusBar outputStatusBar = OutputStatusBar.ON)
        {
            var timestamp = DateTime.Now.ToString("yyyy/MM/dd HH:mm:ss.fffffff");
            var line = $"[{timestamp}] {message}";

            if (outputStatusBar == OutputStatusBar.ON)
            {
                MainWindow.Instance.StatusBar.Text = line;
            }

            Messages += line + "\r\n";
            LogText.Text  += line + "\r\n";
            LogText.Select(LogText.Text.Length, 0);
            Logging(line);
        }

        /// <summary>
        /// ログ画面表示時のイベントハンドラです
        /// </summary>
        /// <param name="sender">イベント送信元インスタンス</param>
        /// <param name="e">イベント引数</param>
        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            // メイン画面の最前面化
            MainWindow.Instance.Activate();

            Title += $" [{MainWindow.LogPath}]";
        }

        /// <summary>
        /// ログウィンドウが閉じるときのイベントハンドラです
        /// </summary>
        /// <param name="sender">イベント送信元インスタンス</param>
        /// <param name="e">イベント引数</param>
        private void Window_Closed(object sender, EventArgs e)
        {
            MainWindow.LogView = null;
        }

        /// <summary>
        /// ログ出力します
        /// </summary>
        /// <param name="message">ログメッセージ</param>
        private void Logging(string message)
        {
            try
            {
                using (var writer = new StreamWriter(MainWindow.LogPath, true))
                {
                    writer.WriteLine(message);
                    writer.Flush();
                    writer.Close();
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show($"ログファイルの出力に失敗しました: {ex.Message}\n{MainWindow.LogPath}",
                    MainWindow.ErrorCaption, MessageBoxButton.OK, MessageBoxImage.Error);
            }
        }
    }
}
